package Net::Cisco::ACS::Device;
use strict;
use Moose;
use Data::Dumper;

BEGIN {
    use Exporter ();
    use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS %actions);
    $VERSION     = '0.01';
    @ISA         = qw(Exporter);
    @EXPORT      = qw();
    @EXPORT_OK   = qw();
    %EXPORT_TAGS = ();
};

    %actions = (	"query" => "/Rest/NetworkDevice/Device",
					"create" => "/Rest/NetworkDevice/Device",
               		"update" => "/Rest/NetworkDevice/Device",
                	"getByName" => "/Rest/NetworkDevice/Device/name/",
                	"getById" => "/Rest/NetworkDevice/Device/id/",
           ); 

# MOOSE!		   

sub BUILD # after Method Modifier for constructor!
{ my $self = shift;
  # Some magic here
}
	   
has 'description' => (
      is  => 'rw',
      isa => 'Any',
  );

has 'id' => (
      is  => 'rw',
      isa => 'Str',
  );

has 'name' => (
	is => 'rw',
	isa => 'Str',
	);

has 'tacacsConnection' => (
	is => 'rw',
	isa => 'HashRef',
	trigger => \&parseTacacsConnection,
	auto_deref => 1,
	);

sub parseTacacsConnection
{ my $self = shift;
  my $new_ref = shift;
  $self->singleConnect($self->tacacsConnection->{"singleConnect"});
  $self->tacacs_SharedSecret($self->tacacsConnection->{"sharedSecret"});
  $self->legacyTACACS($self->tacacsConnection->{"legacyTACACS"});
};

has 'groupInfo' => ( 
	isa => 'Any',
	is => 'rw',
	trigger => \&parseGroupInfo,
	);

sub parseGroupInfo # STILL TO DO: Implement non-default groups!
{ my $self = shift;
  my $new_ref = shift;
  for my $entry (@{ $new_ref })
  { if ($entry->{"groupType"} eq "Location")
    { $self->location($entry->{"groupName"});
    }
    if ($entry->{"groupType"} eq "Device Type")
    { $self->deviceType($entry->{"groupName"});
    }
  }
}
	
has 'legacyTACACS' => (
	is => 'rw',
	isa => 'Str',
	);

has 'tacacs_SharedSecret' => (
	is => 'rw',
	isa => 'Str',
	);

has 'singleConnect' => (
	is => 'rw',
	isa => 'Str',
	);

has 'radius_SharedSecret' => (
	is => 'rw',
	isa => 'Str',
	);

has 'subnets' => (
	is => 'rw',
	isa => 'Any',
	trigger => \&parseSubnet, # trigger modifier is calling within constructor
	);

sub parseSubnet
{ my $self = shift;
  my $new_ref = shift;
  my @ips = ();
  if (ref($new_ref) eq "HASH")
  { push(@ips,{ netMask => $new_ref->{"netMask"}, ipAddress => $new_ref->{"ipAddress"} }); }
  if (ref($new_ref) eq "ARRAY")
  { for my $entry (@ { $new_ref })
    { if (ref($entry) eq "HASH")
	  { push(@ips,{ netMask => $entry->{"netMask"}, ipAddress => $entry->{"ipAddress"} }); } 
	}
  }
  $self->ips([@ips]);
}

has 'ips' => (
	is => 'rw',
	isa => 'ArrayRef',
	auto_deref => 1,
	);
	
has 'location' => (
	is => 'rw',
	isa => 'Str',
	);

has 'deviceType' => (
	is => 'rw',
	isa => 'Str',
	);

has 'displayedInHex' => (
	is => 'rw',
	isa => 'Str',
	);

has 'keyWrap' => (
	is => 'rw',
	isa => 'Str',
	);

has 'portCOA' => (
	is => 'rw',
	isa => 'Str',
	);

# No Moose	

sub toXML
{ my $self = shift;
  my $result = "";
  my $id = $self->id;
  my $description = $self->description || "";
  my $name = $self->name || "";
  my $location = $self->location || "All Locations";
  my $devicetype = $self->deviceType || "All Device Types";

  my $legacytacacs = $self->legacyTACACS || "false";
  my $tacacs_sharedsecret = $self->tacacs_SharedSecret || "";  
  my $singleconnect = $self->singleConnect || "false";

  my $displayedinhex = $self->displayedInHex || "true";
  my $keywrap = $self->keyWrap || "false";
  my $portcoa = $self->portCOA || "1700";
  my $radius_sharedsecret = $self->radius_SharedSecret || "";

  $result = <<XML;
	<description>$description</description>
	<name>$name</name>
	<groupInfo>
	<groupName>$devicetype</groupName>
	<groupType>Device Type</groupType>
	</groupInfo>
	<groupInfo>
	<groupName>$location</groupName>
	<groupType>Location</groupType>
	</groupInfo>
XML

  if (ref($self->ips) eq "ARRAY")
  { for my $ref ( @{ $self->ips } )
    { my $netmask = $ref->{'netMask'};
	  my $ipaddress = $ref->{'ipAddress'};
	  $result .= <<XML;
	<subnets><ipAddress>$ipaddress</ipAddress><netMask>$netmask</netMask></subnets>
XML
	}
  }

  if ($tacacs_sharedsecret) {
  $result .= <<XML;
	<tacacsConnection>
	<legacyTACACS>$legacytacacs</legacyTACACS>
	<sharedSecret>$tacacs_sharedsecret</sharedSecret>
	<singleConnect>$singleconnect</singleConnect>
	</tacacsConnection>
XML
  }

  if ($radius_sharedsecret)
  { $result .= <<XML;
	<radiusConnection>
	<displayedInHex>$displayedinhex</displayedInHex>
	<keyWrap>$keywrap</keyWrap>
	<portCoA>$portcoa</portCoA>
	<sharedSecret>$radius_sharedsecret</sharedSecret>
	</radiusConnection>
XML
  }

  return $result;
}

sub header
{ my $self = shift;
  my $devices = shift;
  return qq(<?xml version="1.0" encoding="UTF-8" standalone="yes"?><ns1:device xmlns:ns1="networkdevice.rest.mgmt.acs.nm.cisco.com">$devices</ns1:device>);
}
	
=head1 NAME

Net::Cisco::ACS::Device - Access Cisco ACS functionality through REST API - Device fields

=head1 SYNOPSIS

  use Net::Cisco::ACS::Device;


=head1 DESCRIPTION

The Net::Cisco::ACS::Device class holds all the device relevant information from Cisco ACS 5.x

=head1 USAGE



=head1 BUGS



=head1 SUPPORT



=head1 AUTHOR

    Hendrik Van Belleghem
    CPAN ID: BEATNIK
    hendrik.vanbelleghem@gmail.com

=head1 COPYRIGHT

This program is free software licensed under the...

	The General Public License (GPL)
	Version 2, June 1991

The full text of the license can be found in the
LICENSE file included with this module.


=head1 SEE ALSO

perl(1).

=cut

#################### main pod documentation end ###################
__PACKAGE__->meta->make_immutable();

1;
# The preceding line will help the module return a true value

